@@ -50,6 +50,21 @@ class EFAwsResolver(object):
   # dictionary of boto3 clients: {"ec2":ec2_client, ... } made with ef_utils.create_aws_clients
   __CLIENTS = {}
 
+  def _elbv2_load_balancer(self, lookup):
+    """
+    Args:
+      lookup: the friendly name of the V2 elb to look up
+    Returns:
+      A dict with the load balancer description
+    Raises:
+      botocore.exceptions.ClientError: no such load-balancer
+    """
+    client = EFAwsResolver.__CLIENTS['elbv2']
+    elbs = client.describe_load_balancers(Names=[lookup])
+    # getting the first one, since we requested only one lb
+    elb = elbs['LoadBalancers'][0]
+    return elb
+
   def acm_certificate_arn(self, lookup, default=None):
     """
     Args:
@@ -83,7 +98,7 @@ class EFAwsResolver(object):
       if cert_handle["DomainName"] == domain_name:
         cert = acm_client.describe_certificate(CertificateArn=cert_handle["CertificateArn"])["Certificate"]
         # Patch up cert if there is no IssuedAt (i.e. cert was not issued by Amazon)
-        if not cert.has_key("IssuedAt"):
+        if "IssuedAt" not in cert:
           cert[u"IssuedAt"] = datetime.datetime(1970, 1, 1, 0, 0)
         if best_match_cert is None:
           best_match_cert = cert
@@ -312,6 +327,127 @@ class EFAwsResolver(object):
     else:
       return default
 
+  def ec2_vpc_endpoint_id(self, lookup, default=None):
+    """
+    Args:
+      lookup: the name of the VPC endpoint to look up (in tags)
+      default: the optional value to return if lookup failed; returns None if not set
+    Returns:
+      The ID of the VPC Endpoint found with a label matching 'lookup' or default/None if no match found
+    """
+    vpc_endpoints = EFAwsResolver.__CLIENTS["ec2"].describe_vpc_endpoints(Filters=[{
+      "Name": "tag:Name",
+      "Values": [lookup]
+    }])
+    if len(vpc_endpoints.get("VpcEndpoints")) > 0:
+      return vpc_endpoints["VpcEndpoints"][0]["VpcEndpointId"]
+    else:
+      return default
+
+  def ec2_vpc_endpoint_id_by_vpc_service(self, lookup, default=None):
+    """
+    Args:
+      lookup: a forward-slash-delimited string of [vpc-name, service-name] "vpc-name/service-name"
+      default: the optional value to return if lookup failed; returns None if not set
+    Returns:
+      The ID of the OLDEST VPC endpoint found in the given VPC for the given service
+    """
+    vpc_name, service_name = lookup.split("/")
+    vpc_id = self.ec2_vpc_vpc_id(vpc_name)
+    if vpc_id is None:
+      return default
+
+    vpc_endpoints = EFAwsResolver.__CLIENTS["ec2"].describe_vpc_endpoints(Filters=[
+      {"Name": "vpc-id", "Values":[vpc_id]},
+      {"Name": "service-name", "Values":[service_name]}
+    ])
+    if len(vpc_endpoints.get("VpcEndpoints")) < 1:
+      return default
+
+    oldest = None
+    for vpce in vpc_endpoints.get("VpcEndpoints"):
+      if oldest is None or vpce["CreationTimestamp"] < oldest["CreationTimestamp"]:
+        oldest = vpce
+    return oldest["VpcEndpointId"]
+
+  def ec2_vpc_vpn_gateway_id(self, lookup, default=None):
+    """
+    Args:
+      lookup: the friendly name of the VPN Gateway ID to look up
+      default: the optional value to return if lookup failed; returns None if not set
+    Returns:
+      The ID of the VPN Gateway found with a label matching 'lookup' or default/None if no match found
+    """
+    vpn_gateways = EFAwsResolver.__CLIENTS["ec2"].describe_vpn_gateways(Filters=[{
+      "Name": "tag:Name",
+      "Values": [lookup]
+    }])
+    if len(vpn_gateways) > 0:
+      return vpn_gateways["VpnGateways"][0]["VpnGatewayId"]
+    else:
+      return default
+
+  def elbv2_load_balancer_hosted_zone(self, lookup, default=None):
+    """
+    Args:
+      lookup: the friendly name of the V2 elb to look up
+      default: value to return in case of no match
+    Returns:
+      The hosted zone ID of the ELB found with a name matching 'lookup'.
+    """
+    try:
+      elb = self._elbv2_load_balancer(lookup)
+      return elb['CanonicalHostedZoneId']
+    except ClientError:
+      return default
+
+  def elbv2_load_balancer_dns_name(self, lookup, default=None):
+    """
+    Args:
+      lookup: the friendly name of the V2 elb to look up
+      default: value to return in case of no match
+    Returns:
+      The hosted zone ID of the ELB found with a name matching 'lookup'.
+    """
+    try:
+      elb = self._elbv2_load_balancer(lookup)
+      return elb['DNSName']
+    except ClientError:
+      return default
+
+  def elbv2_load_balancer_arn_suffix(self, lookup, default=None):
+    """
+    Args:
+      lookup: the friendly name of the v2 elb to look up
+      default: value to return in case of no match
+    Returns:
+      The shorthand fragment of the ALB's ARN, of the form `app/*/*`
+    """
+    try:
+      elb = self._elbv2_load_balancer(lookup)
+      m = re.search(r'.+?(app\/[^\/]+\/[^\/]+)$', elb['LoadBalancerArn'])
+      return m.group(1)
+    except ClientError:
+      return default
+
+  def elbv2_target_group_arn_suffix(self, lookup, default=None):
+    """
+    Args:
+      lookup: the friendly name of the v2 elb target group
+      default: value to return in case of no match
+    Returns:
+      The shorthand fragment of the target group's ARN, of the form
+      `targetgroup/*/*`
+    """
+    try:
+      client = EFAwsResolver.__CLIENTS['elbv2']
+      elbs = client.describe_target_groups(Names=[lookup])
+      elb = elbs['TargetGroups'][0]
+      m = re.search(r'.+?(targetgroup\/[^\/]+\/[^\/]+)$', elb['TargetGroupArn'])
+      return m.group(1)
+    except ClientError:
+      return default
+
   def waf_rule_id(self, lookup, default=None):
     """
     Args:
@@ -327,7 +463,7 @@ class EFAwsResolver(object):
       for rule in rules["Rules"]:
         if rule["Name"] == lookup:
           return rule["RuleId"]
-      if rules.has_key("NextMarker"):
+      if "NextMarker" in rules:
         rules = EFAwsResolver.__CLIENTS["waf"].list_rules(Limit=list_limit, NextMarker=rules["NextMarker"])
       else:
         return default
@@ -347,7 +483,7 @@ class EFAwsResolver(object):
       for acl in acls["WebACLs"]:
         if acl["Name"] == lookup:
           return acl["WebACLId"]
-      if acls.has_key("NextMarker"):
+      if "NextMarker" in acls:
         acls = EFAwsResolver.__CLIENTS["waf"].list_web_acls(Limit=list_limit, NextMarker=acls["NextMarker"])
       else:
         return default
@@ -366,7 +502,7 @@ class EFAwsResolver(object):
       return default
     hosted_zones = EFAwsResolver.__CLIENTS["route53"].list_hosted_zones_by_name(DNSName=lookup, MaxItems=list_limit)
     # Return if the account has no HostedZones
-    if not hosted_zones.has_key("HostedZones"):
+    if "HostedZones" not in hosted_zones:
       return default
     while True:
       for hosted_zone in hosted_zones["HostedZones"]:
@@ -392,7 +528,7 @@ class EFAwsResolver(object):
       return default
     hosted_zones = EFAwsResolver.__CLIENTS["route53"].list_hosted_zones_by_name(DNSName=lookup, MaxItems=list_limit)
     # Return if the account has no HostedZones
-    if not hosted_zones.has_key("HostedZones"):
+    if "HostedZones" not in hosted_zones:
       return default
     while True:
       for hosted_zone in hosted_zones["HostedZones"]:
@@ -451,7 +587,7 @@ class EFAwsResolver(object):
     list_limit = "100"
     distributions = EFAwsResolver.__CLIENTS["cloudfront"].list_distributions(MaxItems=list_limit)["DistributionList"]
     # Return if the account has no Distributions
-    if not distributions.has_key("Items"):
+    if "Items" not in distributions:
       return default
     while True:
       for distribution in distributions["Items"]:
@@ -476,7 +612,7 @@ class EFAwsResolver(object):
     oais = EFAwsResolver.__CLIENTS["cloudfront"].list_cloud_front_origin_access_identities(
       MaxItems=list_limit)["CloudFrontOriginAccessIdentityList"]
     # Return if the account has no OriginAccessIdentities
-    if not oais.has_key("Items"):
+    if "Items" not in oais:
       return default
     while True:
       for oai in oais["Items"]:
@@ -501,7 +637,7 @@ class EFAwsResolver(object):
     oais = EFAwsResolver.__CLIENTS["cloudfront"].list_cloud_front_origin_access_identities(
       MaxItems=list_limit)["CloudFrontOriginAccessIdentityList"]
     # Return if the account has no OriginAccessIdentities
-    if not oais.has_key("Items"):
+    if "Items" not in oais:
       return default
     while True:
       for oai in oais["Items"]:
@@ -513,6 +649,100 @@ class EFAwsResolver(object):
       else:
         return default
 
+  def cognito_identity_identity_pool_arn(self, lookup, default=None):
+    """
+    Args:
+        lookup: Cognito Federated Identity name, proto0-cms-identity-pool
+        default: the optional value to return if lookup failed; returns None if not set
+
+    Returns:
+        the constructed ARN for the cognito identity pool, else default/None
+    """
+    identity_pool_id = self.cognito_identity_identity_pool_id(lookup, default)
+
+    if identity_pool_id == default:
+      return default
+
+    # The ARN has to be constructed because there is no boto3 call that returns the full ARN for a cognito identity pool
+    return "arn:aws:cognito-identity:{{{{REGION}}}}:{{{{ACCOUNT}}}}:identitypool/{}".format(identity_pool_id)
+
+  def cognito_identity_identity_pool_id(self, lookup, default=None):
+    """
+    Args:
+        lookup: Cognito Federated Identity name, proto0-cms-identity-pool
+        default: the optional value to return if lookup failed; returns None if not set
+
+    Returns:
+        the Cognito Identity Pool ID corresponding to the given lookup, else default/None
+    """
+    # List size cannot be greater than 60
+    list_limit = 60
+    client = EFAwsResolver.__CLIENTS["cognito-identity"]
+    response = client.list_identity_pools(MaxResults=list_limit)
+
+    while "IdentityPools" in response:
+      # Loop through all the identity pools
+      for pool in response["IdentityPools"]:
+        if pool["IdentityPoolName"] == lookup:
+          return pool["IdentityPoolId"]
+
+      # No match found on this page, but there are more pages
+      if "NextToken" in response:
+        response = client.list_identity_pools(MaxResults=list_limit, NextToken=response["NextToken"])
+      else:
+        break
+
+    return default
+
+  def cognito_idp_user_pool_arn(self, lookup, default=None):
+    """
+    Args:
+        lookup: Cognito User Pool name, proto0-cms-user-pool
+        default: the optional value to return if lookup failed; returns None if not set
+
+    Returns:
+        the User Pool ARN corresponding to the given lookup, else default/None
+    """
+    client = EFAwsResolver.__CLIENTS["cognito-idp"]
+    user_pool_id = self.cognito_idp_user_pool_id(lookup, default)
+    if user_pool_id == default:
+      return default
+
+    response = client.describe_user_pool(UserPoolId=user_pool_id)
+
+    if "UserPool" not in response:
+      return default
+
+    return response["UserPool"]["Arn"]
+
+  def cognito_idp_user_pool_id(self, lookup, default=None):
+    """
+    Args:
+        lookup: Cognito User Pool name, proto0-cms-user-pool
+        default: the optional value to return if lookup failed; returns None if not set
+
+    Returns:
+        the User Pool ID corresponding to the given lookup, else default/None
+    """
+    # List size cannot be greater than 60
+    list_limit = 60
+    client = EFAwsResolver.__CLIENTS["cognito-idp"]
+    response = client.list_user_pools(MaxResults=list_limit)
+
+    while "UserPools" in response:
+      # Loop through all user pools
+      for pool in response["UserPools"]:
+        if pool["Name"] == lookup:
+          return pool["Id"]
+
+      # No match found on this page, but there are more pages
+      if "NextToken" in response:
+        response = client.list_identity_pools(MaxResults=list_limit, NextToken=response["NextToken"])
+      else:
+        break
+
+    return default
+
   def kms_decrypt_value(self, lookup):
     """
     Args:
@@ -521,7 +751,7 @@ class EFAwsResolver(object):
       The decrypted lookup value
     """
     decrypted_lookup = ef_utils.kms_decrypt(EFAwsResolver.__CLIENTS["kms"], lookup)
-    return decrypted_lookup
+    return decrypted_lookup.plaintext.decode('string_escape')
 
   def kms_key_arn(self, lookup):
     """
@@ -533,6 +763,73 @@ class EFAwsResolver(object):
     key_arn = ef_utils.kms_key_arn(EFAwsResolver.__CLIENTS["kms"], lookup)
     return key_arn
 
+  def ram_resource_share_arn(self, lookup, default=None):
+    """
+    Args:
+      lookup: the name of the resource share to look up
+      default: the optional value to return if lookup failed; returns None if not set
+    Returns:
+      The arn of the first resource share found with a label matching 'lookup' or default/None if no match found
+    """
+    resource_shares = EFAwsResolver.__CLIENTS["ram"].get_resource_shares(
+      resourceOwner="OTHER-ACCOUNTS",
+      name=lookup)
+
+    if len(resource_shares.get("resourceShares")) > 0:
+      return resource_shares["resourceShares"][0]["resourceShareArn"]
+    else:
+      return default
+
+  def ram_resource_arn(self, lookup, default=None):
+    """
+    Args:
+      lookup: the resource share arn to look up
+      default: the optional value to return if lookup failed; returns None if not set
+    Returns:
+      The arn of the first resource found with a label matching 'lookup' or default/None if no match found
+    """
+    resources = EFAwsResolver.__CLIENTS["ram"].list_resources(
+      resourceOwner="OTHER-ACCOUNTS",
+      resourceShareArns=[lookup])
+
+    if len(resources.get("resources")) > 0:
+      return resources["resources"][0]["arn"]
+    else:
+      return default
+
+  def ec2_transit_gateway_id(self, lookup, default=None):
+    """
+    Args:
+      lookup: the arn of the transit gateway to look up
+      default: the optional value to return if lookup failed; returns None if not set
+    Returns:
+      The id of the first transit gateway found with a label matching 'lookup' or default/None if no match found
+    """
+    transit_gateways = EFAwsResolver.__CLIENTS["ec2"].describe_transit_gateways()
+
+    if len(transit_gateways.get("TransitGateways")) > 0:
+      transit_gateways = [i for i in transit_gateways["TransitGateways"] if (i["TransitGatewayArn"] == lookup)]
+      return transit_gateways[0]["TransitGatewayId"]
+    else:
+      return default
+
+  def ecr_repository_uri(self, lookup, default=None):
+    """
+    Args:
+      lookup: the name of the Docker image to look up
+      default: the optional value to return if lookup failed; returns None if not set
+    Returns:
+      The id of the first image found with a label matching 'lookup' or default/None if no match found
+    """
+    try:
+      repositories = EFAwsResolver.__CLIENTS["ecr"].describe_repositories(repositoryNames=[ lookup ])
+      if len(repositories.get("repositories")) > 0:
+        return repositories.get("repositories")[0]["repositoryUri"]
+      else:
+        return default
+    except ClientError:
+      return default
+
   def lookup(self, token):
     try:
       kv = token.split(",")
@@ -546,6 +843,14 @@ class EFAwsResolver(object):
       return self.cloudfront_origin_access_identity_oai_canonical_user_id(*kv[1:])
     elif kv[0] == "cloudfront:origin-access-identity/oai-id":
       return self.cloudfront_origin_access_identity_oai_id(*kv[1:])
+    elif kv[0] == "cognito-identity:identity-pool-arn":
+      return self.cognito_identity_identity_pool_arn(*kv[1:])
+    elif kv[0] == "cognito-identity:identity-pool-id":
+      return self.cognito_identity_identity_pool_id(*kv[1:])
+    elif kv[0] == "cognito-idp:user-pool-arn":
+      return self.cognito_idp_user_pool_arn(*kv[1:])
+    elif kv[0] == "cognito-idp:user-pool-id":
+      return self.cognito_idp_user_pool_id(*kv[1:])
     elif kv[0] == "ec2:elasticip/elasticip-id":
       return self.ec2_elasticip_elasticip_id(*kv[1:])
     elif kv[0] == "ec2:elasticip/elasticip-ipaddress":
@@ -564,6 +869,8 @@ class EFAwsResolver(object):
       return self.ec2_subnet_subnet_cidr(*kv[1:])
     elif kv[0] == "ec2:subnet/subnet-id":
       return self.ec2_subnet_subnet_id(*kv[1:])
+    elif kv[0] == "ec2:transit-gateway/transit-gateway-id":
+      return self.ec2_transit_gateway_id(*kv[1:])
     elif kv[0] == "ec2:vpc/availabilityzones":
       return self.ec2_vpc_availabilityzones(*kv[1:])
     elif kv[0] == "ec2:vpc/cidrblock":
@@ -572,10 +879,30 @@ class EFAwsResolver(object):
       return self.ec2_vpc_subnets(*kv[1:])
     elif kv[0] == "ec2:vpc/vpc-id":
       return self.ec2_vpc_vpc_id(*kv[1:])
+    elif kv[0] == "ec2:vpc-endpoint/vpc-endpoint-id":
+      return self.ec2_vpc_endpoint_id(*kv[1:])
+    elif kv[0] == "ec2:vpc-endpoint/vpc-endpoint-id/by-vpc-service":
+      return self.ec2_vpc_endpoint_id_by_vpc_service(*kv[1:])
+    elif kv[0] == "ec2:vpc/vpn-gateway-id":
+      return self.ec2_vpc_vpn_gateway_id(*kv[1:])
+    elif kv[0] == "ecr:repository/repository-uri":
+      return self.ecr_repository_uri(*kv[1:])
+    elif kv[0] == "elbv2:load-balancer/dns-name":
+      return self.elbv2_load_balancer_dns_name(*kv[1:])
+    elif kv[0] == "elbv2:load-balancer/hosted-zone":
+      return self.elbv2_load_balancer_hosted_zone(*kv[1:])
+    elif kv[0] == "elbv2:load-balancer/arn-suffix":
+      return self.elbv2_load_balancer_arn_suffix(*kv[1:])
+    elif kv[0] == "elbv2:target-group/arn-suffix":
+      return self.elbv2_target_group_arn_suffix(*kv[1:])
     elif kv[0] == "kms:decrypt":
       return self.kms_decrypt_value(*kv[1:])
     elif kv[0] == "kms:key_arn":
       return self.kms_key_arn(*kv[1:])
+    elif kv[0] == "ram:resource-share/resource-share-arn":
+      return self.ram_resource_share_arn(*kv[1:])
+    elif kv[0] == "ram:resource-share/resource-arn":
+      return self.ram_resource_arn(*kv[1:])
     elif kv[0] == "route53:private-hosted-zone-id":
       return self.route53_private_hosted_zone_id(*kv[1:])
     elif kv[0] == "route53:public-hosted-zone-id":
