@@ -30,7 +30,6 @@ limitations under the License.
 from __future__ import print_function
 import argparse
 import json
-from os import getenv
 from os.path import dirname, normpath
 import sys
 import time
@@ -40,10 +39,10 @@ from botocore.exceptions import ClientError
 from ef_aws_resolver import EFAwsResolver
 from ef_config import EFConfig
 from ef_context import EFContext
-from ef_plugin import run_plugins
 from ef_service_registry import EFServiceRegistry
 from ef_template_resolver import EFTemplateResolver
-from ef_utils import create_aws_clients, fail, get_account_id, http_get_metadata, pull_repo
+from ef_utils import create_aws_clients, fail, get_account_id, http_get_metadata
+from ef_conf_utils import pull_repo
 
 # Globals
 CLIENTS = None
@@ -77,7 +76,7 @@ SERVICE_TYPE_ROLE = {
   "aws_ec2": "ec2.amazonaws.com",
   "aws_lambda": "lambda.amazonaws.com",
   "aws_role": None, # Bare role requires a custom assume role policy ("assume_role_policy": "name_of_policy")
-  "http_service": "ec2.amazonaws.com"
+  "http_service": ["ec2.amazonaws.com", "ecs.amazonaws.com", "ecs-tasks.amazonaws.com"]
 }
 
 # these service types get Security Groups
@@ -97,6 +96,7 @@ INSTANCE_PROFILE_SERVICE_TYPES = [
 # these service types get KMS Keys
 KMS_SERVICE_TYPES = [
   "aws_ec2",
+  "aws_fixture",
   "aws_lambda",
   "http_service"
 ]
@@ -110,10 +110,13 @@ def handle_args_and_set_context(args):
     a populated EFContext object
   Raises:
     IOError: if service registry file can't be found or can't be opened
-    RuntimeError: if repo or branch isn't as spec'd in ef_config.EF_REPO and ef_config.EF_REPO_BRANCH
+    RuntimeError: if branch isn't as spec'd in ef_config.EF_REPO_BRANCH
     CalledProcessError: if 'git rev-parse' command to find repo root could not be run
   """
-  parser = argparse.ArgumentParser()
+  parser = argparse.ArgumentParser(
+    description="Generate a security group, IAM role, instance profile, attache managed, custom, or inline "
+                "generated policies to a role, create KMS keys, and create newrelic alerts for a service based "
+                "on the information specified in the service_registry.json.")
   parser.add_argument("env", help=", ".join(EFConfig.ENV_LIST))
   parser.add_argument("--sr", help="optional /path/to/service_registry_file.json", default=None)
   parser.add_argument("--commit", help="Make changes in AWS (dry run if omitted)", action="store_true", default=False)
@@ -127,7 +130,7 @@ def handle_args_and_set_context(args):
   try:
     context.env = parsed_args["env"]
   except ValueError as e:
-    fail("Error in env: {}".format(e.message))
+    fail("Error in env: {}".format(e))
   # Set up service registry and policy template path which depends on it
   context.service_registry = EFServiceRegistry(parsed_args["sr"])
   context.policy_template_path = normpath(dirname(context.service_registry.filespec)) + EFConfig.POLICY_TEMPLATE_PATH_SUFFIX
@@ -278,7 +281,7 @@ def conditionally_create_role(role_name, sr_entry):
     print_if_verbose("not eligible for role (and possibly instance profile); service type: {}".format(service_type))
     return
 
-  if sr_entry.has_key("assume_role_policy"):
+  if "assume_role_policy" in sr_entry:
     # Explicitly defined AssumeRole policy
     assume_role_policy_document = resolve_policy_document(sr_entry["assume_role_policy"])
   else:
@@ -286,15 +289,16 @@ def conditionally_create_role(role_name, sr_entry):
     # which must list a service type to use this capacity (most do)
     if SERVICE_TYPE_ROLE[service_type] is None:
       fail("service_type: {} does not have a default service-type AssumeRole policy".format(service_type))
-    formatted_principals = '"Service": "{}"'.format(SERVICE_TYPE_ROLE[service_type])
-    assume_role_policy_document = '''{
+    service_principals = {"Service": SERVICE_TYPE_ROLE[service_type]}
+    assume_role_policy = {
       "Version" : "2012-10-17",
       "Statement": [{
         "Effect": "Allow",
-        "Principal": { ''' + formatted_principals + ''' },
+        "Principal": service_principals,
         "Action": [ "sts:AssumeRole" ]
       }]
-    }'''
+    }
+    assume_role_policy_document = json.dumps(assume_role_policy, indent=2)
   if not get_role_id(role_name):
     print("Create role: {}".format(role_name))
     print_if_verbose("AssumeRole policy document:\n{}".format(assume_role_policy_document))
@@ -340,7 +344,7 @@ def conditionally_create_profile(role_name, service_type):
   else:
     print_if_verbose("instance profile already contains role: {}".format(role_name))
 
-def conditionally_attach_managed_policies(role_name, sr_entry):
+def conditionally_attach_aws_managed_policies(role_name, sr_entry):
   """
   If 'aws_managed_policies' key lists the names of AWS managed policies to bind to the role,
   attach them to the role
@@ -360,6 +364,40 @@ def conditionally_attach_managed_policies(role_name, sr_entry):
     if CONTEXT.commit:
       try:
         CLIENTS["iam"].attach_role_policy(RoleName=role_name, PolicyArn='arn:aws:iam::aws:policy/' + policy_name)
+        print_if_verbose("Attached managed policy '{}'".format(policy_name))
+      except CLIENTS["iam"].exceptions.NoSuchEntityException:
+        CLIENTS["iam"].attach_role_policy(RoleName=role_name,
+                                          PolicyArn='arn:aws:iam::aws:policy/job-function/' + policy_name)
+        print_if_verbose("Attached managed job-function '{}'".format(policy_name))
+      except:
+        fail("Exception putting policy: {} onto role: {}".format(policy_name, role_name), sys.exc_info())
+
+def conditionally_attach_customer_managed_policies(role_name, sr_entry):
+  """
+  If 'customer_managed_policies' key lists the names of customer managed policies to bind to the role,
+  attach them to the role.
+
+  Note that this function will throw a warning without failing, if the managed policy does not exist in
+  the given account.
+  Args:
+    role_name: name of the role to attach the policies to
+    sr_entry: service registry entry
+  """
+  service_type = sr_entry['type']
+  if not (service_type in SERVICE_TYPE_ROLE and "customer_managed_policies" in sr_entry):
+    print_if_verbose("not eligible for policies; service_type: {} is not valid for policies "
+                     "or no 'customer_managed_policies' key in service registry for this role".format(service_type))
+    return
+
+  for policy_name in sr_entry['customer_managed_policies']:
+    print_if_verbose("loading policy: {} for role: {}".format(policy_name, role_name))
+
+    if CONTEXT.commit:
+      policy_arn = 'arn:aws:iam::{}:policy/{}'.format(CONTEXT.account_id, policy_name)
+      try:
+        CLIENTS["iam"].attach_role_policy(RoleName=role_name, PolicyArn=policy_arn)
+      except CLIENTS["iam"].exceptions.NoSuchEntityException as exc:
+        print("WARNING: {}".format(exc))
       except:
         fail("Exception putting policy: {} onto role: {}".format(policy_name, role_name), sys.exc_info())
 
@@ -396,7 +434,7 @@ def conditionally_create_kms_key(role_name, service_type):
   Create KMS Master Key for encryption/decryption of sensitive values in cf templates and latebind configs
   Args:
       role_name: name of the role that kms key is being created for; it will be given decrypt privileges.
-      service_type: service registry service type: 'aws_ec2', 'aws_lambda', or 'http_service'
+      service_type: service registry service type: 'aws_ec2', 'aws_fixture', 'aws_lambda', or 'http_service'
   """
   if service_type not in KMS_SERVICE_TYPES:
     print_if_verbose("not eligible for kms; service_type: {} is not valid for kms".format(service_type))
@@ -413,55 +451,80 @@ def conditionally_create_kms_key(role_name, service_type):
     else:
       fail("Exception describing KMS key: {} {}".format(role_name, error))
 
-  formatted_principal = '"AWS": "arn:aws:iam::{}:role/{}"'.format(CONTEXT.account_id, role_name)
-  kms_key_policy = '''{
-    "Version": "2012-10-17",
-    "Statement": [
-      {
-        "Sid": "Enable IAM User Permissions",
-        "Effect": "Allow",
-        "Principal": {
-          "AWS": "arn:aws:iam::''' + CONTEXT.account_id + ''':root"
+  account_id = CONTEXT.account_id
+  if service_type == "aws_fixture":
+    kms_key_policy = {
+      "Version": "2012-10-17",
+      "Statement": [
+        {
+          "Sid": "Enable IAM User Permissions",
+          "Effect": "Allow",
+          "Principal": {
+            "AWS": "arn:aws:iam::{}:root".format(account_id)
+          },
+          "Action": "kms:*",
+          "Resource": "*"
+        }
+      ]
+    }
+  else:
+    role_principal = [ "arn:aws:iam::{}:role/{}".format(account_id, role_name) ]
+    if '.' not in role_name:
+      subservice_principals = "arn:aws:iam::{}:role/{}.*".format(account_id, role_name)
+      role_principal.append(subservice_principals)
+    kms_key_policy = {
+      "Version": "2012-10-17",
+      "Statement": [
+        {
+          "Sid": "Enable IAM User Permissions",
+          "Effect": "Allow",
+          "Principal": {
+            "AWS": "arn:aws:iam::{}:root".format(account_id)
+          },
+          "Action": "kms:*",
+          "Resource": "*"
         },
-        "Action": "kms:*",
-        "Resource": "*"
-      },
-      {
-        "Sid": "Allow Service Role Decrypt Privileges",
-        "Effect": "Allow",
-        "Principal": { ''' + formatted_principal + ''' },
-        "Action": "kms:Decrypt",
-        "Resource": "*"
-      },
-      {
-        "Sid": "Allow use of the key for default autoscaling group service role",
-        "Effect": "Allow",
-        "Principal": { "AWS": "arn:aws:iam::''' + CONTEXT.account_id + ''':role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling" },
-        "Action": [
-          "kms:Encrypt",
-          "kms:Decrypt",
-          "kms:ReEncrypt*",
-          "kms:GenerateDataKey*",
-          "kms:DescribeKey"
-        ],
-        "Resource": "*"
-      },
-      {
-        "Sid": "Allow attachment of persistent resourcesfor default autoscaling group service role",
-        "Effect": "Allow",
-        "Principal": { "AWS": "arn:aws:iam::''' + CONTEXT.account_id + ''':role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling" },
-        "Action": [
-          "kms:CreateGrant"
-        ],
-        "Resource": "*",
-        "Condition": {
-          "Bool": {
-            "kms:GrantIsForAWSResource": true
+        {
+          "Sid": "Allow Service Role Decrypt Privileges",
+          "Effect": "Allow",
+          "Principal": "*",
+          "Action": "kms:Decrypt",
+          "Resource": "*",
+          "Condition": {
+                "ForAnyValue:StringLike": {
+                    "aws:PrincipalArn": role_principal
+                }
+            }
+        },
+        {
+          "Sid": "Allow use of the key for default autoscaling group service role",
+          "Effect": "Allow",
+          "Principal": { "AWS": "arn:aws:iam::{}:role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling".format(account_id) },
+          "Action": [
+            "kms:Encrypt",
+            "kms:Decrypt",
+            "kms:ReEncrypt*",
+            "kms:GenerateDataKey*",
+            "kms:DescribeKey"
+          ],
+          "Resource": "*"
+        },
+        {
+          "Sid": "Allow attachment of persistent resourcesfor default autoscaling group service role",
+          "Effect": "Allow",
+          "Principal": { "AWS": "arn:aws:iam::{}:role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling".format(account_id) },
+          "Action": [
+            "kms:CreateGrant"
+          ],
+          "Resource": "*",
+          "Condition": {
+            "Bool": {
+              "kms:GrantIsForAWSResource": True
+            }
           }
         }
-      }
-    ]
-  }'''
+      ]
+    }
 
   if not kms_key:
     print("Create KMS key: {}".format(key_alias))
@@ -472,7 +535,7 @@ def conditionally_create_kms_key(role_name, service_type):
       while create_key_failures <= 5:
         try:
           new_kms_key = CLIENTS["kms"].create_key(
-            Policy=kms_key_policy,
+            Policy=json.dumps(kms_key_policy, indent=4),
             Description='Master Key for {}'.format(role_name)
           )
           break
@@ -497,11 +560,22 @@ def conditionally_create_kms_key(role_name, service_type):
   else:
     print_if_verbose("KMS key already exists: {}".format(key_alias))
 
+
+def create_newrelic_alerts():
+  """
+   Create Newrelic Alerts for each entry in the service registry application_services
+   Note: Import is inside this function rather than top of the file so that we do not import when running unit tests.
+         (would otherwise require an ef_site_config.yml in the directory where tests are being run)
+  """
+  from newrelic_executor import NewRelicAlerts
+  print("> Creating NewRelic Alerts")
+  NewRelicAlerts(CONTEXT, CLIENTS).run()
+
 def main():
   global CONTEXT, CLIENTS, AWS_RESOLVER
 
   CONTEXT = handle_args_and_set_context(sys.argv[1:])
-  if not (CONTEXT.devel or getenv("JENKINS_URL", False)):
+  if not CONTEXT.devel and CONTEXT.whereami != 'jenkins':
     try:
       pull_repo()
     except RuntimeError as error:
@@ -513,11 +587,11 @@ def main():
   try:
     # If running in EC2, always use instance credentials. One day we'll have "lambda" in there too, so use "in" w/ list
     if CONTEXT.whereami == "ec2":
-      CLIENTS = create_aws_clients(EFConfig.DEFAULT_REGION, None, "ec2", "iam", "kms")
+      CLIENTS = create_aws_clients(EFConfig.DEFAULT_REGION, None, "cloudfront", "ec2", "iam", "kms")
       CONTEXT.account_id = str(json.loads(http_get_metadata('iam/info'))["InstanceProfileArn"].split(":")[4])
     else:
       # Otherwise, we use local user creds based on the account alias
-      CLIENTS = create_aws_clients(EFConfig.DEFAULT_REGION, CONTEXT.account_alias, "ec2", "iam", "kms", "sts")
+      CLIENTS = create_aws_clients(EFConfig.DEFAULT_REGION, CONTEXT.account_alias, "cloudfront", "ec2", "iam", "kms", "sts")
       CONTEXT.account_id = get_account_id(CLIENTS["sts"])
   except RuntimeError:
     fail("Exception creating AWS clients in region {} with profile {}".format(
@@ -568,17 +642,21 @@ def main():
     # 3. KMS KEY FOR THE SERVICE : only some types of services get kms keys
     conditionally_create_kms_key(target_name, service_type)
 
-    # 4 ATTACH AWS MANAGED POLICIES TO ROLE
-    conditionally_attach_managed_policies(target_name, sr_entry)
+    # 4. ATTACH AWS MANAGED POLICIES TO ROLE
+    conditionally_attach_aws_managed_policies(target_name, sr_entry)
 
-    # 5. INLINE SERVICE'S POLICIES INTO ROLE
+    # 5. ATTACH CUSTOMER MANAGED POLICIES TO ROLE
+    conditionally_attach_customer_managed_policies(target_name, sr_entry)
+
+    # 6. INLINE SERVICE'S POLICIES INTO ROLE
     # only eligible service types with "policies" sections in the service registry get policies
     conditionally_inline_policies(target_name, sr_entry)
 
-  # 5. Execute plugins
-  run_plugins(context_obj=CONTEXT, boto3_clients=CLIENTS)
+  # Create newrelic alerts for all "application_services" in the service registry
+  create_newrelic_alerts()
 
   print("Exit: success")
 
+
 if __name__ == "__main__":
   main()
