@@ -28,7 +28,6 @@ import argparse
 from inspect import isfunction
 import json
 from operator import itemgetter
-from os import getenv
 import sys
 import urllib2
 
@@ -37,7 +36,8 @@ from botocore.exceptions import ClientError
 from ef_config import EFConfig
 from ef_context import EFContext
 from ef_service_registry import EFServiceRegistry
-from ef_utils import create_aws_clients, fail, pull_repo
+from ef_utils import create_aws_clients, fail
+from ef_conf_utils import pull_repo
 from ef_version_resolver import EFVersionResolver
 
 VERBOSE = False
@@ -56,6 +56,7 @@ class EFVersionContext(EFContext):
     self._limit = None
     self._noprecheck = None
     self._rollback = None
+    self._rollback_to = ""
     self._service_name = None # Cheating - we don't care about the full service record so don't use context.service
     self._show = None
     self._stable = None
@@ -131,6 +132,10 @@ class EFVersionContext(EFContext):
   def rollback(self):
     return self._rollback
 
+  @property
+  def rollback_to(self):
+    return self._rollback_to
+
   @property
   def service_name(self):
     """The service being queried or set (name only, since gets don't use the service registry"""
@@ -172,29 +177,16 @@ class Version(object):
   """
 
   def __init__(self, object_version):
-    if EFConfig.S3_VERSION_BUILDNUMBER_KEY in object_version["Metadata"]:
-      self._build_number = object_version["Metadata"][EFConfig.S3_VERSION_BUILDNUMBER_KEY]
-    else:
-      self._build_number = ""
-    if EFConfig.S3_VERSION_COMMITHASH_KEY in object_version["Metadata"]:
-      self._commit_hash = object_version["Metadata"][EFConfig.S3_VERSION_COMMITHASH_KEY]
-    else:
-      self._commit_hash = ""
-    self._last_modified = object_version["LastModified"].strftime("%Y-%m-%dT%H:%M:%S%Z")
-    if EFConfig.S3_VERSION_LOCATION_KEY in object_version["Metadata"]:
-      self._location = object_version["Metadata"][EFConfig.S3_VERSION_LOCATION_KEY]
-    else:
-      self._location = ""
-    if EFConfig.S3_VERSION_MODIFIEDBY_KEY in object_version["Metadata"]:
-      self._modified_by = object_version["Metadata"][EFConfig.S3_VERSION_MODIFIEDBY_KEY]
-    else:
-      self._modified_by = ""
-    if EFConfig.S3_VERSION_STATUS_KEY in object_version["Metadata"]:
-      self._status = object_version["Metadata"][EFConfig.S3_VERSION_STATUS_KEY]
-    else:
-      self._status = ""
     self._value = object_version["Body"].read()
     self._version_id = object_version["VersionId"]
+    self._last_modified = object_version["LastModified"].strftime("%Y-%m-%dT%H:%M:%S%Z")
+
+    metadata = object_version["Metadata"]
+    self._build_number = metadata.get(EFConfig.S3_VERSION_BUILDNUMBER_KEY,"")
+    self._commit_hash = metadata.get(EFConfig.S3_VERSION_COMMITHASH_KEY,"")
+    self._location = metadata.get(EFConfig.S3_VERSION_LOCATION_KEY,"")
+    self._modified_by = metadata.get(EFConfig.S3_VERSION_MODIFIEDBY_KEY,"")
+    self._status = metadata.get(EFConfig.S3_VERSION_STATUS_KEY,"")
 
   def __str__(self):
     return "{} {} {} {} {} {} {} {}".format(self._value, self._build_number, self._commit_hash, self._last_modified,
@@ -267,7 +259,8 @@ def handle_args_and_set_context(args):
   Returns:
     a populated EFVersionContext object
   """
-  parser = argparse.ArgumentParser()
+  parser = argparse.ArgumentParser(description="Perform version tracking of a key for a service in the "
+                                               "service_registry.json")
   parser.add_argument("service_name", help="name of the service")
   parser.add_argument("key", help="version key to look up for <service_name> such as 'ami-id' (list in EF_Config)")
   parser.add_argument("env", help=", ".join(EFConfig.ENV_LIST))
@@ -276,6 +269,8 @@ def handle_args_and_set_context(args):
   group.add_argument("--set", help="set current version of <key> to <value> for <service_name>")
   group.add_argument("--rollback", help="set current version to most recent 'stable' version in history",
                      action="store_true")
+  group.add_argument("--rollback-to", help="rollback current version to <ami-id> in history",
+                     action="store", metavar='<ami-id>')
   group.add_argument("--history", help="Show version history for env/service/key", choices=['json', 'text'])
   group.add_argument("--show", help="Show keys and values. '*' allowed for <key> and <env>",
                      action="store_true", default=False)
@@ -311,7 +306,7 @@ def handle_args_and_set_context(args):
   try:
     context.env = parsed_args["env"]
   except ValueError as e:
-    fail("Error in env: {}".format(e.message))
+    fail("Error in env: {}".format(e))
   # marshall this module's additional context values
   context._get = parsed_args["get"]
   context._history = parsed_args["history"]
@@ -323,6 +318,7 @@ def handle_args_and_set_context(args):
   context._limit = parsed_args["limit"]
   context._location = parsed_args["location"]
   context._rollback = parsed_args["rollback"]
+  context._rollback_to = parsed_args["rollback_to"]
   context._service_name = parsed_args["service_name"]
   context._show = parsed_args["show"]
   context._stable = parsed_args["stable"]
@@ -345,23 +341,27 @@ def print_if_verbose(message):
 
 def validate_context(context):
   """
-    Set the key for the current context.
+    Validate the context. Fails the process on an invalid context
     Args:
       context: a populated EFVersionContext object
   """
-  # Service must exist in service registry
-  if not context.service_registry.service_record(context.service_name):
-    fail("service: {} not found in service registry: {}".format(
-         context.service_name, context.service_registry.filespec))
-  service_type = context.service_registry.service_record(context.service_name)["type"]
 
   # Key must be valid
-  if context.key not in EFConfig.VERSION_KEYS:
+  key_data = EFConfig.VERSION_KEYS.get(context.key)
+  if not key_data:
     fail("invalid key: {}; see VERSION_KEYS in ef_config for supported keys".format(context.key))
 
+  registry = context.service_registry
+  service = registry.service_record(context.service_name)
+  # Service must exist in service registry
+  if not service:
+    fail("service: {} not found in service registry: {}".format(
+         context.service_name, registry.filespec))
+
   # Lookup allowed key for service type
-  if "allowed_types" in EFConfig.VERSION_KEYS[context.key] and \
-          service_type not in EFConfig.VERSION_KEYS[context.key]["allowed_types"]:
+  service_type = service["type"]
+  allowed_types = key_data.get("allowed_types", [])
+  if service_type not in allowed_types:
     fail("service_type: {} is not allowed for key {}; see VERSION_KEYS[KEY]['allowed_types']"
          "in ef_config and validate service registry entry".format(service_type, context.key))
 
@@ -495,21 +495,47 @@ def precheck(context):
     return True
 
 
-def get_versions(context, return_stable=False):
+def _get_stable_versions(context):
+  """
+  Get all stable versions
+  Args:
+    context: a populated EFVersionContext object
+
+  Returns:
+    List of Version objects representing stable versions in this service's history
+  """
+  versions = get_versions(context)
+  stable_versions = []
+  for version in versions:
+    if version.status == EFConfig.S3_VERSION_STATUS_STABLE:
+      stable_versions.append(version)
+  return stable_versions
+
+
+def _get_latest_version(context):
+  """
+  Get latest version in history of service
+  Args:
+    context: a populated EFVersionContext object
+
+  Returns:
+    A Version object representing the latest version in the service's history
+  """
+  original_limit = context.limit
+  context.limit = 1
+  latest_version = get_versions(context)
+  context.limit = original_limit
+  return latest_version[0] if len(latest_version) > 0 else None
+
+
+def get_versions(context):
   """
   Get all versions of a key
   Args:
     context: a populated EFVersionContext object
     return_stable: (default:False) If True, stop fetching if 'stable' version is found; return only that version
   Returns:
-    json list of object data sorted in reverse by last_modified (newest version is first). Each item is a dict:
-    {
-      'value': <value>,
-      'last_modified": <YYYY-MM-DDThh:mm:ssZ>, (ISO8601 date time string)
-      'modified_by': '<arn:aws:...>',
-      'version_id': '<version_id>',
-      'status': See EF_Config.S3_VERSION_STATUS_* for possible values
-    }
+    List of Version objects sorted in reverse by last_modified (newest version is first).
   """
   s3_key = "{}/{}/{}".format(context.service_name, context.env, context.key)
   object_version_list = context.aws_client("s3").list_object_versions(
@@ -518,8 +544,10 @@ def get_versions(context, return_stable=False):
       MaxKeys=context.limit,
       Prefix=s3_key
   )
+
   if "Versions" not in object_version_list:
     return []
+
   object_versions = []
   for version in object_version_list["Versions"]:
     object_version = Version(context.aws_client("s3").get_object(
@@ -527,16 +555,9 @@ def get_versions(context, return_stable=False):
         Key=s3_key,
         VersionId=version["VersionId"]
     ))
-    # Stop if a stable version was found and return_stable was set
-    if return_stable and object_version.status == EFConfig.S3_VERSION_STATUS_STABLE:
-      return [object_version]
     object_versions.append(object_version)
 
-  # If caller is looking for a 'stable' version and we made it to here, a stable version was not found
-  if return_stable:
-    return []
-  else:
-    return sorted(object_versions, key=lambda v: v.last_modified, reverse=True)
+  return sorted(object_versions, key=lambda v: v.last_modified, reverse=True)
 
 
 def cmd_get(context):
@@ -554,6 +575,23 @@ def cmd_history(context):
     print(json.dumps(versions, cls=VersionEncoder))
 
 
+def get_version_by_value(context, value):
+  """
+  Get the latest version that matches the provided ami-id
+  Args:
+    context: a populated EFVersionContext object
+    value: the value of the version to look for
+  """
+  versions = get_versions(context)
+  for version in versions:
+    if version.value == value:
+      return version
+  fail("Didn't find a matching version for: "
+       "{}:{} in env/service: {}/{}".format(
+          context.key, value,
+          context.env, context.service_name))
+
+
 def cmd_rollback(context):
   """
   Roll back by finding the most recent "stable" tagged version, and putting it again, so that
@@ -561,20 +599,40 @@ def cmd_rollback(context):
   Args:
     context: a populated EFVersionContext object
   """
-  last_stable = get_versions(context, return_stable=True)
-  if len(last_stable) != 1:
-    fail("Didn't find a version marked stable for key: {} in env/service: {}/{}".format(
-         context.key, context.env, context.service_name))
-  context.value = last_stable[0].value
-  context.commit_hash = last_stable[0].commit_hash
-  context.build_number = last_stable[0].build_number
-  context.location = last_stable[0].location
+  stable_versions = _get_stable_versions(context)
+  latest_version = _get_latest_version(context)
+  for version in stable_versions:
+    if latest_version and (version.value != latest_version.value):
+      context.value = version.value
+      context.commit_hash = version.commit_hash
+      context.build_number = version.build_number
+      context.location = version.location
+      context.stable = True
+      cmd_set(context)
+      return
+  fail("Didn't find a version marked stable for key: {} in env/service: {}/{}".format(context.key, context.env,
+                                                                                      context.service_name))
+
+
+def cmd_rollback_to(context):
+  """
+  Roll back by finding a specific version in the history of the service and
+  putting it as the new current version.
+  Args:
+    context: a populated EFVersionContext object
+  """
+  version = get_version_by_value(context, context.rollback_to)
+  context.value = version.value
+  context.commit_hash = version.commit_hash
+  context.build_number = version.build_number
+  context.location = version.location
   context.stable = True
   cmd_set(context)
 
 
-def _getlatest_ami_id(context):
+def _get_deployed_ami_id(context):
   """
+  NOTE: Currently not used until the future
   Get the most recent AMI ID for a service
   Args:
     context: a populated EFVersionContext object
@@ -629,7 +687,7 @@ def cmd_set(context):
   try:
     precheck(context)
   except Exception as e:
-    fail("Precheck failed: {}".format(e.message))
+    fail("Precheck failed: {}".format(e))
 
   s3_key = "{}/{}/{}".format(context.service_name, context.env, context.key)
   s3_version_status = EFConfig.S3_VERSION_STATUS_STABLE if context.stable else EFConfig.S3_VERSION_STATUS_UNDEFINED
@@ -677,7 +735,7 @@ def main():
   context = handle_args_and_set_context(sys.argv[1:])
 
   # Refresh from repo if necessary and possible (gets don't need service registry, sets do)
-  if (context.rollback or context.value) and not (context.devel or getenv("JENKINS_URL", False)):
+  if (context.rollback or context.value) and (not context.devel and context.whereami != 'jenkins'):
     print("Refreshing repo")
     try:
       pull_repo()
@@ -708,6 +766,8 @@ def main():
     cmd_history(context)
   elif context.rollback:
     cmd_rollback(context)
+  elif context.rollback_to:
+    cmd_rollback_to(context)
   elif context.show:
     cmd_show(context)
   elif context.value:
